#!/usr/bin/env python
import ConfigParser
import logging
import subprocess
import SocketServer
import socket

import cmefpf_pb2

CI_TOOL = "./tools/ci_tool"
CONFIG_FILE = "agent.conf"

class CmeHandler(SocketServer.StreamRequestHandler):
    
    store = {}

    def handle(self):
        logging.info("CME connected")
        while self.rfile:
            # read header and payload
            t = self.rfile.read(2)
            if not t:
                break
            length = (ord(t[0]) << 8) | ord(t[1])
            data = self.rfile.read(length)
            if len(data) != length:
                logging.debug("length too short {0} < {1}".format(
                        len(data), length))
                break
            # decode payload
            m = cmefpf_pb2.GenericRequest.FromString(data)
            logging.debug("message {0}".format(m))
            if m.type == m.CONFIGURE_STREAM:
                r = self.configure_stream(m)
            elif m.type == m.COLLECT_EXPIRED_STREAMS:
                r = self.collect_expired_streams(m)
            else:
                break
            length = r.ByteSize()
            assert(length < (1 << 16))
            self.wfile.write(''.join([
                        chr(length >> 8),
                        chr(length & 0xff),
                        r.SerializeToString()]))
        logging.info("CME disconnected")

    def configure_stream(self, m):
        m = m.configure
        r = cmefpf_pb2.GenericResponse()
        r.type = r.CONFIGURE_STREAM_RESULT
        r.configure.id = m.id
        # call ci_tool
        key = "".join(["%02x" % ord(i) for i in m.key])
        params = [CI_TOOL, "add", str(m.id), str(m.refresh_time),
                  str(m.filter.ip_source), str(m.filter.ip_destination),
                  str(m.filter.protocol), str(m.filter.port_source),
                  str(m.filter.port_destination), key]
        logging.debug("Calling: " + str(params))
        try:
            p = subprocess.Popen(params)
            p.wait()
        except Exception, e:
            logging.debug("Result: starting ci_tool failed" + str(e))
            r.configure.result = r.configure.FPF_FAILURE
            r.configure.description = "Starting ci_tool failed."
            return r
        logging.debug("Result: " + str(p.returncode))
        # send reponse
        if p.returncode:
            r.configure.result = r.configure.FPF_FAILURE
        else:
            self.store[m.id] = m
            r.configure.result = r.configure.FPF_SUCCESS
        return r

    def collect_expired_streams(self, m):
        r = cmefpf_pb2.GenericResponse()
        r.type = r.COLLECT_EXPIRED_STREAMS_RESULT
        # call ci_tool
        params = [CI_TOOL, "collect"]
        logging.debug("Calling: " + str(params))
        try:
            p = subprocess.Popen(params, stdout=subprocess.PIPE)
            p.wait()
            for line in p.stdout:
                logging.debug("* " + line.strip())
                id, duration, bytes = line.split()
                id, duration, bytes = int(id), int(duration), int(bytes)
                cm = self.store.get(id, None)
                if not cm:
                    continue
                stream = r.collect.id.add()
                stream.id = id
                stream.filter.ip_source = cm.filter.ip_source
                stream.filter.ip_destination = cm.filter.ip_destination
                stream.filter.protocol = cm.filter.protocol
                stream.filter.port_source = cm.filter.port_source
                stream.filter.port_destination = cm.filter.port_destination
                stream.bandwidth = cm.bandwidth
                stream.cos = cm.cos
                stream.as_path.extend(cm.as_path)
                stream.transferred_bytes = bytes
                stream.duration = duration
                del self.store[id]
        except Exception, e:
            logging.warn("Result: starting ci_tool failed, " + str(e))
        finally:
            return r



if __name__ == "__main__":
    conf = ConfigParser.ConfigParser()
    conf.read(CONFIG_FILE)
    HOST = conf.get("fpf agent", "address")
    PORT = conf.getint("fpf agent", "port")
    is_debug_enabled = conf.getboolean("fpf agent", "debug")
    if is_debug_enabled:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    addressType = socket.getaddrinfo(HOST, PORT)
    if len(addressType[0][4]) == 4:
		# IPv6 address
		SocketServer.TCPServer.address_family = socket.AF_INET6
	
    SocketServer.BaseServer.allow_reuse_address = True
    server = SocketServer.TCPServer( (HOST, PORT), CmeHandler)
    server.serve_forever()
