/*
 * FPF implementation.
 * 2013 @ Jaroslaw Sliwinski  &  Piotr Wisniewski
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>

#include <linux/netfilter.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Jaroslaw Sliwinski    Piotr Wisniewski");
MODULE_DESCRIPTION("FPF intercept module. It mathcoes IP flows and assigns COMET delivery header.");

#include "ci_netfilter.h"
#include "ci_netlink.h"
#include "ci_store.h"

static int __init ci_init(void)
{
	printk(KERN_INFO "FPF intercept: starting\n");
	if (ci_create_store() < 0)
		goto error0;
	if (ci_register_netfilter() < 0)
		goto error1;   
	if (ci_register_netlink() < 0)
		goto error2;
	return 0;
error2:
	ci_deregister_netfilter();
error1:
	ci_destroy_store();
error0:
	printk(KERN_WARNING "FPF intercept: failed to start\n");
	return -ENOMEM;
}

static void __exit ci_exit(void)
{
	printk(KERN_INFO "FPF intercept: closing\n");
	ci_deregister_netlink();
	ci_deregister_netfilter();
	ci_destroy_store();
}

module_init(ci_init);
module_exit(ci_exit);

